/**********************************************************
 SixXS - Automatic IPv6 Connectivity Configuration Utility
***********************************************************
 Copyright 2003-2005 SixXS - http://www.sixxs.net
***********************************************************
 common/aiccu.c - AICCU Abstracted functions
***********************************************************
 $Author: jeroen $
 $Id: aiccu.c,v 1.4 2005/01/11 12:16:09 jeroen Exp $
 $Date: 2005/01/11 12:16:09 $
**********************************************************/

#include "aiccu.h"

struct AICCU_conf *g_aiccu = NULL;

/* Config */
struct pl_rule aiccu_conf_rules[] =
{
	/* Configuration */
	{"username",		PLRT_STRING,	offsetof(struct AICCU_conf, username)},
	{"password",		PLRT_STRING,	offsetof(struct AICCU_conf, password)},
	{"ipv4_interface",	PLRT_STRING,	offsetof(struct AICCU_conf, ipv4_interface)},
	{"ipv6_interface",	PLRT_STRING,	offsetof(struct AICCU_conf, ipv6_interface)},
	{"tunnel_id",		PLRT_STRING,	offsetof(struct AICCU_conf, tunnel_id)},

	/* Automatic */
	{"automatic",		PLRT_BOOL,	offsetof(struct AICCU_conf, automatic)},

	/* Operational options */
	{"daemonize",		PLRT_BOOL,	offsetof(struct AICCU_conf, daemonize)},
	{"verbose",		PLRT_BOOL,	offsetof(struct AICCU_conf, verbose)},
	{NULL,			PLRT_END,	0},
};

bool aiccu_InitConfig()
{
	/* Allocate & Initialize */
	g_aiccu = (struct AICCU_conf *)malloc(sizeof(*g_aiccu));
	if (!g_aiccu) return false;
	memset(g_aiccu, 0, sizeof(*g_aiccu));
	g_aiccu->tic = (struct TIC_conf *)malloc(sizeof(*g_aiccu->tic));
	memset(g_aiccu->tic, 0, sizeof(*g_aiccu->tic));

	/* Initialize config to defaults */
	g_aiccu->daemonize	= -1;
	g_aiccu->verbose	= false;
	g_aiccu->ipv6_interface	= strdup("aiccu");

	return true;
}

/* configure this client */
bool aiccu_LoadConfig(const char *filename)
{
	FILE			*f;
	char			buf[1000];
	unsigned int		line = 0;

	f = fopen(filename, "r");
	if (!f)
	{
		dolog(LOG_ERR, "Could not open config file \"%s\"\n", filename);
		return false;
	}

	while (fgets(buf, sizeof(buf), f))
	{
		line++;
		if (parseline(buf, " ", aiccu_conf_rules, g_aiccu)) continue;

		dolog(LOG_WARNING, "Unknown configuration statement on line %d: \"%s\"\n", line, buf);
	}
	fclose(f);
	return true;
}

/* Save the configuration */
bool aiccu_SaveConfig(const char *filename)
{
	FILE *f = fopen(filename, "w");
	if (!f)
	{
		dolog(LOG_ERR, "Could not open config file \"%s\" for writing\n", filename);
		return false;
	}

	fprintf(f, "# AICCU Configuration (Automatically stored by aiccu %s)\n", AICCU_VER);
	fprintf(f, "\n");
	fprintf(f, "# Login information\n");
	fprintf(f, "username %s\n", g_aiccu->username);
	fprintf(f, "password %s\n", g_aiccu->password);
	fprintf(f, "\n");
	fprintf(f, "# Interface names to use\n");
	fprintf(f, "ipv4_interface %s\n", g_aiccu->ipv4_interface);
	fprintf(f, "ipv6_interface %s\n", g_aiccu->ipv6_interface);
	fprintf(f, "\n");
	fprintf(f, "# The tunnel_id to use\n");
	fprintf(f, "# (only required when there are multiple tunnels in the list)\n");
	fprintf(f, "tunnel_id %s\n", g_aiccu->tunnel_id);
	fprintf(f, "\n");
	fprintf(f, "automatic %s\n", g_aiccu->automatic ? "true" : "false");
	fprintf(f, "\n");
	fprintf(f, "# Be verbose?\n");
	fprintf(f, "verbose %s\n", g_aiccu->verbose ? "true" : "false");
	fprintf(f, "\n");
	fprintf(f, "# Daemonize?\n");
	fprintf(f, "daemonize %s\n", g_aiccu->daemonize ? "true" : "false");
	fprintf(f, "\n");
	fprintf(f, "# Behind a NAT?\n");
	fprintf(f, "behindnat %s\n", g_aiccu->behindnat ? "true" : "false");
	fclose(f);
	return true;
}

void aiccu_FreeConfig()
{
	if (!g_aiccu) return;
	if (g_aiccu->username)		free(g_aiccu->username);
	g_aiccu->username = NULL;
	if (g_aiccu->password)		free(g_aiccu->password);
	g_aiccu->password = NULL;
	if (g_aiccu->ipv4_interface)	free(g_aiccu->ipv4_interface);
	g_aiccu->ipv4_interface = NULL;
	if (g_aiccu->ipv6_interface)	free(g_aiccu->ipv6_interface);
	g_aiccu->ipv6_interface = NULL;
	if (g_aiccu->tunnel_id)		free(g_aiccu->tunnel_id);
	g_aiccu->tunnel_id = NULL;
	if (g_aiccu->tic)		free(g_aiccu->tic);
	g_aiccu->tic = NULL;
	free(g_aiccu);
	g_aiccu = NULL;
}

/* OS Interfaces - quick and easy */
void aiccu_os_install();
void aiccu_os_setup(struct TIC_Tunnel *hTunnel);
void aiccu_os_reconfig(struct TIC_Tunnel *hTunnel);
void aiccu_os_delete(struct TIC_Tunnel *hTunnel);
void aiccu_os_test(struct TIC_Tunnel *hTunnel, bool automatic);

/* Make sure the OS understands IPv6 */
void aiccu_install()
{
	D(dolog(LOG_DEBUG, "aiccu_install()\n");)
	aiccu_os_install();
}

void aiccu_setup(struct TIC_Tunnel *hTunnel, bool firstrun)
{
	D(dolog(LOG_DEBUG, "aiccu_setup(%s, %s)\n", hTunnel->sIPv6_Local, firstrun ? "first" : "other");)

	/* AYIYA calls aiccu_setup(hTunnel,false) after preparing the tunnel interface */
	if (firstrun && strcasecmp(hTunnel->sType, "ayiya") == 0)
	{
		ayiya(hTunnel);
	}
	else aiccu_os_setup(hTunnel);
}

void aiccu_reconfig(struct TIC_Tunnel *hTunnel)
{
	D(dolog(LOG_DEBUG, "aiccu_reconfig(%s)\n", hTunnel->sIPv6_Local);)
	aiccu_os_reconfig(hTunnel);
}

void aiccu_delete(struct TIC_Tunnel *hTunnel)
{
	D(dolog(LOG_DEBUG, "aiccu_delete(%s)\n", hTunnel->sIPv6_Local);)
	aiccu_os_delete(hTunnel);
}

void aiccu_test(struct TIC_Tunnel *hTunnel, bool automatic)
{
	D(dolog(LOG_DEBUG, "aiccu_delete(%s)\n", hTunnel->sIPv6_Local);)
	aiccu_os_test(hTunnel, automatic);
}

void aiccu_exec(char *fmt, ...)
{
#ifndef _WIN32
	char buf[1024];
	va_list ap;
	va_start(ap, fmt);
	vsnprintf(buf,sizeof(buf),fmt,ap);
	D(dolog(LOG_DEBUG, "aiccu_os_exec(\"%s\")\n", buf));
	system(buf);
	va_end(ap);
#endif
}

#define AICCU_LICENSE_PART1 "\
Copyright (C) SixXS\n\
All rights reserved.\n\
\n\
Redistribution and use in source and binary forms, with or without\n\
modification, are permitted provided that the following conditions\n\
are met:\n\
1. Redistributions of source code must retain the above copyright\n\
   notice, this list of conditions and the following disclaimer.\n"

#define AICCU_LICENSE_PART2 "\
2. Redistributions in binary form must reproduce the above copyright\n\
   notice, this list of conditions and the following disclaimer in the\n\
   documentation and/or other materials provided with the distribution.\n\
3. Neither the name of SixXS nor the names of its contributors\n\
   may be used to endorse or promote products derived from this software\n\
   without specific prior permission.\n"

#define AICCU_LICENSE_PART3 "\
4. One should not remove any reference to, or logo of, SixXS.\n\
5. When the software is altered to not use SixXS services, one is kindly\n\
   asked to notify SixXS of this by sending an email to the SixXS Staff\n\
   at info@sixxs.net, containing at least the following details:\n\
   8<--------\n\
   Organisation : Organisation Name\n\
   Email        : mailbox@example.com\n\
   Website      : http://www.example.com\n\
   is using software <Software Name> for:\n"

#define AICCU_LICENSE_PART4 "\
   <\n\
    description of:\n\
       the usage\n\
       the reason why it was modified\n\
   >\n\
   -------->8\n\
   Additional information details may of course be provided.\n\
   We request this to be able to know why people would choose not to\n\
   use the services provided by SixXS and the participating ISP's.\n\
   \n"

#define AICCU_LICENSE_PART5 "\
THIS SOFTWARE IS PROVIDED BY SIXXS AND CONTRIBUTORS ``AS IS'' AND\n\
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\n\
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE\n\
ARE DISCLAIMED.  IN NO EVENT SHALL SIXXS OR CONTRIBUTORS BE LIABLE\n\
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\n\
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS\n"

#define AICCU_LICENSE_PART6 "\
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)\n\
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT\n\
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY\n\
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF\n\
SUCH DAMAGE.\n"

const char *aiccu_license()
{
#ifndef NOPEDANTIC
	/*
	* Pedantic doesn't allow this long strings, thus we will
	 * play nice and malloc it, copy them in seperatly and
	 * then return the buffer.
	 * What we don't do for compliancy....
	*/
	static char *license = NULL;
	if (!license)
	{
		/*
		 * Make one big block out of it
		 * too bad that the \0's get inserted,
		 * remove them and tada one big text...
		*/
		static char
			l1[] = AICCU_LICENSE_PART1,
			l2[] = AICCU_LICENSE_PART2,
			l3[] = AICCU_LICENSE_PART3,
			l4[] = AICCU_LICENSE_PART4,
			l5[] = AICCU_LICENSE_PART5,
			l6[] = AICCU_LICENSE_PART6;
		size_t
			a = strlen(l1),
			b = strlen(l2),
			c = strlen(l3),
			d = strlen(l4),
			e = strlen(l5),
			f = strlen(l6);

		/* Create the 'long' string our selves then */
		license = (char *)malloc(a+b+c+d+e+f);
		if (!license) return NULL;

		memcpy(license    , l1, a);
		memcpy(license + a, l2, b);
		memcpy(license + a + b, l3, c);
		memcpy(license + a + b + c, l4, d);
		memcpy(license + a + b + c + d, l5, e);
		memcpy(license + a + b + c + d + e, l6, f);
	}
	return license;
#else
	return AICCU_LICENSE_PART1 AICCU_LICENSE_PART2 AICCU_LICENSE_PART3
		AICCU_LICENSE_PART4 AICCU_LICENSE_PART5 AICCU_LICENSE_PART6;
#endif
}

