/**********************************************************
 SixXS - Automatic IPv6 Connectivity Configuration Utility
***********************************************************
 Copyright 2003-2005 SixXS - http://www.sixxs.net
***********************************************************
 common/aiccu.h - Abstracted Functions and Configuration
 All compile-time configurable items are in this file
***********************************************************
 $Author: jeroen $
 $Id: aiccu.h,v 1.4 2005/01/31 19:05:52 jeroen Exp $
 $Date: 2005/01/31 19:05:52 $
**********************************************************/

#ifndef AICCU_H
#define AICCU_H "H5K7:W3NDY5UU5N1K1N1C0l3"

#include "common.h"
#include "tic.h"
#include "heartbeat.h"
#include "ayiya.h"

/* AICCU Version */
#define AICCU_VER		"2005.01.31"
#define AICCU_VERSION_NUM	2005,01,31,0
/* _NUM = required for Windows Resources */

/* Append -gui or -console? */
#ifndef AICCU_CONSOLE
#ifdef AICCU_TYPE
#define AICCU_VERSION AICCU_VER "-gui-" AICCU_TYPE
#else
#define AICCU_VERSION AICCU_VER "-gui"
#endif
#else
#ifdef AICCU_TYPE
#define AICCU_VERSION AICCU_VER "-console-" AICCU_TYPE
#else
#define AICCU_VERSION AICCU_VER "-console"
#endif
#endif

/* Needed for TIC */
#define TIC_CLIENT_NAME "AICCU"
#define TIC_CLIENT_VERSION AICCU_VERSION

/* 
 * AICCU configuration Cache
 * allows reconnects even when we don't update
 * the data. Could be useful in the event
 * where we can't make contact to the main server
 */
#define AICCU_CACHE	"/var/cache/aiccu.cache"

/* The PID we are running as when daemonized */
#define AICCU_PID	"/var/run/aiccu.pid"

/* AICCU Configuration file */
#ifdef _WIN32
/* GetWindowsDirectory() is used to figure out the directory to store the config */
#define AICCU_CONFIG	"aiccu.conf"
#else
#define AICCU_CONFIG	"/etc/aiccu.conf"
#endif

/* Inbound listen queue */
#define LISTEN_QUEUE    128

/* AICCU Configuration */
struct AICCU_conf
{
	/* used by other parts */
	int		daemonize;
	bool		verbose;

	/* Only for AICCU */
	char		*username;
	char		*password;
	char		*ipv4_interface;
	char		*ipv6_interface;
	char		*tunnel_id;

	bool		automatic;		/* Try to be totally automatic? */

	bool		behindnat;		/* Behind a NAT */

	struct TIC_conf	*tic;			/* TIC Structure */
};

/* Global configuration */
extern struct AICCU_conf *g_aiccu;

/* AICCU Abstracted Functions */
bool aiccu_InitConfig(void);
bool aiccu_LoadConfig(const char *filename);
bool aiccu_SaveConfig(const char *filename);
void aiccu_FreeConfig();

void aiccu_install();
void aiccu_setup(struct TIC_Tunnel *hTunnel, bool firstrun);
void aiccu_reconfig(struct TIC_Tunnel *hTunnel);
void aiccu_delete(struct TIC_Tunnel *hTunnel);
void aiccu_test(struct TIC_Tunnel *hTunnel, bool automatic);
void aiccu_exec(char *fmt, ...);
const char *aiccu_license();

#endif /* AICCU_H */

