/**********************************************************
 SixXS - Automatic IPv6 Connectivity Configuration Utility
***********************************************************
 Copyright 2003-2005 SixXS - http://www.sixxs.net
***********************************************************
 common/ayiya.c - AYIYA - Anything In Anything
***********************************************************
 $Author: jeroen $
 $Id: ayiya.c,v 1.6 2005/01/17 13:37:36 jeroen Exp $
 $Date: 2005/01/17 13:37:36 $
**********************************************************/

#include "aiccu.h"
#include "ayiya.h"
#include "tic.h"

struct pseudo_ayh
{
	struct ayiyahdr	ayh;
	struct in6_addr	identity;
	sha1_byte	hash[SHA1_DIGEST_LENGTH];
	unsigned char	payload[2048];
};

struct in_addr		ayiya_ipv4_pop;			/* IPv4 remote endpoint */
struct in6_addr		ayiya_ipv6_local,		/* IPv6 local  endpoint */
			ayiya_ipv6_pop;			/* IPv6 remote endpoint */
sha1_byte		ayiya_hash[SHA1_DIGEST_LENGTH];	/* SHA1 Hash of the shared secret. */

/* The tun/tap device HANDLE */
#ifndef _WIN32
int			ayiya_fd;
#else
HANDLE			device_handle = INVALID_HANDLE_VALUE;
#define ETH_P_IPV6 0x86dd
#define ETH_ALEN 6
struct ether_header
{
  u_int8_t  ether_dhost[ETH_ALEN];      /* destination eth addr */
  u_int8_t  ether_shost[ETH_ALEN];      /* source ether addr    */
  u_int16_t ether_type;                 /* packet type ID field */
};
#endif

SOCKET ayiya_socket = -1;

/* Tap device constants which we use */
#define TAP_CONTROL_CODE(request,method) CTL_CODE(FILE_DEVICE_PHYSICAL_NETCARD | 8000, request, method, FILE_ANY_ACCESS)
#define TAP_IOCTL_CONFIG_POINT_TO_POINT TAP_CONTROL_CODE(6, METHOD_BUFFERED)
#define TAP_IOCTL_SET_MEDIA_STATUS      TAP_CONTROL_CODE(7, METHOD_BUFFERED)

void ayiya_log(int level, struct sockaddr_storage *clientaddr, socklen_t addrlen, const char *fmt, ...)
{
	char	buf[1024];
	char	clienthost[NI_MAXHOST];
	char	clientservice[NI_MAXSERV];
	va_list	ap;

	/* Clear them just in case */
	memset(buf, 0, sizeof(buf));
	memset(clienthost, 0, sizeof(clienthost));
	memset(clientservice, 0, sizeof(clientservice));

	if (clientaddr)
	{
		int ret;
		ret = getnameinfo((struct sockaddr *)clientaddr, addrlen,
			clienthost, sizeof(clienthost),
			clientservice, sizeof(clientservice),
			NI_NUMERICHOST|NI_NUMERICSERV);
		if (ret != 0)
		{
			printf("ret: %d, errno: %u, %s\n", ret, errno, strerror(errno));
		}
	}
	
	/* Print the host+port this is coming from */
	snprintf(buf, sizeof(buf), "[AYIYA]%s%s%s%s : ",
		clientaddr ? " [" : "",
		clientaddr ? clienthost : "" ,
		clientaddr ? "]:" : "",
		clientservice ? clientservice : "");

	/* Print the log message behind it */
	va_start(ap, fmt);
	vsnprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
	va_end(ap);
	
	/* Actually Log it */
	dolog(level, buf);
}

/* Tap -> Socket */
#ifndef _WIN32
void *ayiya_reader(void *arg)
#else
DWORD WINAPI ayiya_reader(LPVOID arg)
#endif
{
	unsigned char		buf[2048];
	struct pseudo_ayh	*s = (struct pseudo_ayh *)buf, s2;
	int			lenout;
	SHA_CTX			sha1;
	sha1_byte		hash[SHA1_DIGEST_LENGTH];
	struct sockaddr_in	target;

#ifdef _WIN32
	DWORD			n;
	OVERLAPPED		overlapped;

	/* Create an event for overlapped results */
	overlapped.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
#else
	ssize_t			n;
	/* use the param and let gcc not complain */
	arg = NULL;
#endif

	ayiya_log(LOG_INFO, NULL, 0, "Writer (TAP to Socket) started\n");

	while (true)
	{
		/* We tunnel over IPv4 */
		memcpy(&target.sin_addr, &ayiya_ipv4_pop, sizeof(target.sin_addr));
		target.sin_family = AF_INET;
		target.sin_port = htons(atoi(AYIYA_PORT));
	
		/* Prefill some standard AYIYA values */
		memset(&s, 0, sizeof(s));
		s2.ayh.ayh_idlen	= 4;			/* 2^4 = 16 bytes = 128 bits (IPv6 address) */
		s2.ayh.ayh_idtype	= ayiya_id_integer;
		s2.ayh.ayh_siglen	= 5;			/* 5*4 = 20 bytes = 160 bits (SHA1) */
		s2.ayh.ayh_hshmeth	= ayiya_hash_sha1;
		s2.ayh.ayh_autmeth	= ayiya_auth_sharedsecret;
		s2.ayh.ayh_opcode	= ayiya_op_forward;
		s2.ayh.ayh_nextheader	= IPPROTO_IPV6;

		/* Our IPv6 side of this tunnel */
		memcpy(&s2.identity, &ayiya_ipv6_local, sizeof(s2.identity));
#ifndef _WIN32	
		n = read(ayiya_fd, s2.payload, sizeof(s2.payload));
		if (n <= 0)
		{
			ayiya_log(LOG_ERR, NULL, 0, "[writer] Error reading from device (%d): %s\n", errno, strerror(errno));
			continue;
		}

#ifndef DARWIN
		if (s2.payload[2] != 0x86 || s2.payload[3] != 0xdd)
		{
			ayiya_log(LOG_ERR, NULL, 0, "[writer] Ignoring %02x%02x packet\n", s2.payload[2], s2.payload[3]);
			continue;
		}
		/* get the tun_pi struct out of there */
		memmove(&s2.payload, &s2.payload[4], n-4);
#endif
#else
		overlapped.Offset = 0;
		overlapped.OffsetHigh = 0;
		ResetEvent(overlapped.hEvent);

		n = ReadFile(device_handle, s2.payload, sizeof(s2.payload), &n, &overlapped);
		if (!n)
		{
			if (GetLastError() == ERROR_IO_PENDING)
			{
				WaitForSingleObject(overlapped.hEvent, INFINITE);
				if (!GetOverlappedResult(device_handle, &overlapped, &n, FALSE)) continue;
			}
			else
			{
				ayiya_log(LOG_ERR, NULL, 0, "[writer] Error reading from device (%d): %s\n", errno, strerror(errno));
				continue;
			}
		}
#endif
		/* Fill in the current time */
		s2.ayh.ayh_epochtime = htonl((u_long)time(NULL));

		/*
		 * The hash of the shared secret needs to be in the
		 * spot where we later put the complete hash
		 */
		memcpy(&s2.hash, ayiya_hash, sizeof(s2.hash));

		/* Generate a SHA1 */
		SHA1_Init(&sha1);
		/* Hash the complete AYIYA packet */
		SHA1_Update(&sha1, (sha1_byte *)&s2, sizeof(s2)-sizeof(s2.payload)+n);
		/* Store the hash in the packets hash */
		SHA1_Final(hash, &sha1);

		/* Store the hash in the actual packet */
		memcpy(&s2.hash, &hash, sizeof(s2.hash));

		/* Send it onto the network */
		n = sizeof(s2)-sizeof(s2.payload)+n;
		lenout = sendto(ayiya_socket, (const char *)&s2, (unsigned int)n, 0, (struct sockaddr *)&target, sizeof(target));
		if (lenout < 0)
		{
			ayiya_log(LOG_ERR, NULL, 0, "[writer] Error (%d) while sending %u bytes sent to network (%d): %s)\n", lenout, n, errno, strerror(errno));
		}
		else if (n != lenout)
		{
			ayiya_log(LOG_ERR, NULL, 0, "[writer] Only %u of %u bytes sent to network (%d): %s)\n", lenout, n, errno, strerror(errno));
		}
	}
#ifndef _WIN32
	return NULL;
#else
	return 0;
#endif
}

/* Socket -> Tap */
#ifndef _WIN32
void *ayiya_writer(void *arg)
#else
DWORD WINAPI ayiya_writer(LPVOID arg)
#endif
{
	unsigned char buf[2048];
	struct pseudo_ayh	*s = (struct pseudo_ayh *)buf;
	struct sockaddr_storage	ci;
	socklen_t		cl;
	int			i, n;
	unsigned int		payloadlen = 0;
	SHA_CTX			sha1;
	sha1_byte		their_hash[SHA1_DIGEST_LENGTH],
				our_hash[SHA1_DIGEST_LENGTH];
#ifdef _WIN32
	DWORD			lenout;
	OVERLAPPED		overlapped = {0};
#else
	/* use the param and let gcc not complain */
	arg = NULL;
#endif

	ayiya_log(LOG_INFO, NULL, 0, "Reader (Socket to TAP) started\n");

	while (true)
	{
		cl = sizeof(ci);
		memset(buf, 0, sizeof(buf));
		n = recvfrom(ayiya_socket, (char *)buf, sizeof(buf), 0, (struct sockaddr *)&ci, &cl);

		if (n < 0) continue;

		if (n < (int)sizeof(struct ayiyahdr))
		{
			ayiya_log(LOG_WARNING, &ci, cl, "[reader] Packet too short");
			continue;
		}

		if (	s->ayh.ayh_idlen != 4 ||
			s->ayh.ayh_idtype != ayiya_id_integer ||
			s->ayh.ayh_siglen != 5 ||
			s->ayh.ayh_hshmeth != ayiya_hash_sha1 ||
			s->ayh.ayh_autmeth != ayiya_auth_sharedsecret ||
			(s->ayh.ayh_nextheader != IPPROTO_IPV6 &&
			 s->ayh.ayh_nextheader != IPPROTO_NONE) ||
			(s->ayh.ayh_opcode != ayiya_op_forward &&
			 s->ayh.ayh_opcode != ayiya_op_echo_request &&
			 s->ayh.ayh_opcode != ayiya_op_echo_request_forward))
		{
			/* Invalid AYIYA packet */
			ayiya_log(LOG_ERR, &ci, cl, "Dropping invalid AYIYA packet\n");
			ayiya_log(LOG_ERR, &ci, cl, "idlen:   %u != %u\n", s->ayh.ayh_idlen, 4);
			ayiya_log(LOG_ERR, &ci, cl, "idtype:  %u != %u\n", s->ayh.ayh_idtype, ayiya_id_integer);
			ayiya_log(LOG_ERR, &ci, cl, "siglen:  %u != %u\n", s->ayh.ayh_siglen, 5);
			ayiya_log(LOG_ERR, &ci, cl, "hshmeth: %u != %u\n", s->ayh.ayh_hshmeth, ayiya_hash_sha1);
			ayiya_log(LOG_ERR, &ci, cl, "autmeth: %u != %u\n", s->ayh.ayh_autmeth, ayiya_auth_sharedsecret);
			ayiya_log(LOG_ERR, &ci, cl, "nexth  : %u != %u || %u\n", s->ayh.ayh_nextheader, IPPROTO_IPV6, IPPROTO_NONE);
			ayiya_log(LOG_ERR, &ci, cl, "opcode : %u != %u || %u || %u\n", s->ayh.ayh_opcode, ayiya_op_forward, ayiya_op_echo_request, ayiya_op_echo_request_forward);
			continue;
		}

		if (memcmp(&s->identity, &ayiya_ipv6_pop, sizeof(s->identity)) != 0)
		{
			memset(buf, 0, sizeof(buf));
			inet_ntop(AF_INET6, &s->identity, (char *)&buf, sizeof(buf));
			ayiya_log(LOG_WARNING, &ci, cl, "[reader] Received packet from a wrong identity \"%s\"\n", buf);
			continue;
		}
	
		/* Verify the epochtime */
		i = tic_checktime(ntohl(s->ayh.ayh_epochtime));
		if (i != 0)
		{
			memset(buf, 0, sizeof(buf));
			inet_ntop(AF_INET6, &s->identity, (char *)&buf, sizeof(buf));
			ayiya_log(LOG_WARNING, &ci, cl, "[reader] Time is %d seconds off for %s\n", i, buf);
			continue;
		}
	
		/* How long is the payload? */
		payloadlen = n - (sizeof(*s) - sizeof(s->payload));
	
		/* Save their hash */
		memcpy(&their_hash, &s->hash, sizeof(their_hash));

		/* Copy in our SHA1 hash */
		memcpy(&s->hash, &ayiya_hash, sizeof(s->hash));

		/* Generate a SHA1 of the header + identity + shared secret */
		SHA1_Init(&sha1);
		/* Hash the Packet */
		SHA1_Update(&sha1, (sha1_byte *)s, n);
		/* Store the hash */
		SHA1_Final(our_hash, &sha1);

		memcpy(&s->hash, &our_hash, sizeof(s->hash));

		/* Compare the SHA1's */
		if (memcmp(&their_hash, &our_hash, sizeof(their_hash)) != 0)
		{
			ayiya_log(LOG_WARNING, &ci, cl, "[reader] Incorrect Hash received\n");
			continue;
		}

		if (s->ayh.ayh_nextheader == IPPROTO_IPV6)
		{
			/* Verify that this is really IPv6 */
			if (s->payload[0] >> 4 != 6)
			{
				ayiya_log(LOG_ERR, &ci, cl, "[reader] Received packet didn't start with a 6, thus is not IPv6\n");
				continue;
			}

			/* Forward the packet to the kernel */
#ifndef _WIN32
			{
				unsigned int c;
#ifdef linux
				struct
				{
					struct tun_pi   pi;
					char            payload[2048];
				} packet;

				memset(&packet, 0, sizeof(packet));

				packet.pi.proto = htons(ETH_P_IPV6);
				memcpy(&packet.payload, &s->payload, payloadlen);

				/* Forward the packet to the kernel */
				c = write(ayiya_fd, &packet, payloadlen+sizeof(struct tun_pi));
				if (c != payloadlen+sizeof(struct tun_pi))
				{
					ayiya_log(LOG_ERR, &ci, cl, "[writer] Error while writing to TAP: %u != %u\n", c, payloadlen+sizeof(struct tun_pi));
				}
#else
				c = write(ayiya_fd, &s->payload, payloadlen);
				if (c != payloadlen)
				{
					ayiya_log(LOG_ERR, &ci, cl, "[writer] Error while writing to TAP: %u != %u\n", c, payloadlen);
				}
#endif

			}
#else
			if (!WriteFile(device_handle, &s->payload, payloadlen, &lenout, &overlapped))
			{
				ayiya_log(LOG_ERR, &ci, cl, "[writer] Error while writing to TAP\n");
			}
#endif
		}
	}
#ifndef _WIN32
	return NULL;
#else
	return 0;
#endif
}

bool ayiya(struct TIC_Tunnel *hTunnel)
{
        SHA_CTX			sha1;
	struct addrinfo hints, *res, *ressave;
#ifndef _WIN32
	pthread_t		thread;
#ifdef linux
	struct ifreq		ifr;

	/* Create a new tap device */
	ayiya_fd = open("/dev/net/tun", O_RDWR);
	if (ayiya_fd == -1)
	{
		ayiya_log(LOG_ERR, NULL, 0, "Couldn't open device %s (%d): %s\n", "/dev/net/tun", errno, strerror(errno));
		return false;
	}

	memset(&ifr, 0, sizeof(ifr));
	/* Request a TUN device */
	ifr.ifr_flags = IFF_TUN;
	/* Set the interface name */
	strncpy(ifr.ifr_name, g_aiccu->ipv6_interface, sizeof(ifr.ifr_name));

	if (ioctl(ayiya_fd, TUNSETIFF, &ifr))
	{
		ayiya_log(LOG_ERR, NULL, 0, "Couldn't set interface name to %s (%d): %s\n",
			g_aiccu->ipv6_interface, errno, strerror(errno));
		return false;
	}
#else
	int i;
	ayiya_fd = -1;
	for (i = 0; i < 256; ++i)
	{
		char buf[128];
		snprintf(buf, sizeof(buf), "/dev/tun%d", i);
		ayiya_fd = open(buf, O_RDWR);
		if (ayiya_fd >= 0)
		{
			break;
		}
	}
	if (ayiya_fd < 0)
	{
		ayiya_log(LOG_ERR, NULL, 0, "Couldn't open device %s (%d): %s\n", "/dev/tun", errno, strerror(errno));
		return false;
	}
#endif /* linux */

#else
	HKEY	key;
	DWORD	pID;
	HANDLE	h;
	int	i;

	char adapterid[1024];
	char tapname[1024];
	DWORD len;
	unsigned long status;

	/* Open registry and look for network adapters */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
		"SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}",
		0, KEY_READ, &key))
	{
		ayiya_log(LOG_ERR, NULL, 0, "Could not open the networking registry key\n");
		return false;
	}

	for (i = 0; device_handle == INVALID_HANDLE_VALUE; i++)
	{
		len = sizeof(adapterid);
		if (RegEnumKeyEx(key, i, adapterid, &len, 0, 0, 0, NULL)) break;

		snprintf(tapname, sizeof(tapname), "\\\\.\\%s.tap", adapterid);
		ayiya_log(LOG_DEBUG, NULL, 0, "Trying %s\n", tapname);
		device_handle = CreateFile(tapname, GENERIC_WRITE | GENERIC_READ, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_SYSTEM | FILE_FLAG_OVERLAPPED, 0);
	}

	RegCloseKey(key);

	if (device_handle == INVALID_HANDLE_VALUE)
	{
		ayiya_log(LOG_ERR, NULL, 0, "No Tap device found!\n");
		return false;
	}

	status = true;
	DeviceIoControl(device_handle, TAP_IOCTL_SET_MEDIA_STATUS, &status, sizeof(status), &status, sizeof(status), &len, NULL);

	{
		u_int32_t ep[2];
		/* Use bogus IP's, we just want to get configured as PtP ;) */
		ep[0] = htonl(0x09044504);
		ep[1] = htonl(0x09044505);
		if (!DeviceIoControl(device_handle, TAP_IOCTL_CONFIG_POINT_TO_POINT,
			ep, sizeof(ep),
			ep, sizeof(ep),
			&len, NULL))
		{
			ayiya_log(LOG_ERR, NULL, 0, "Could not set point to point mode...");
			return false;
		}
	}

#endif /* _WIN32 */
	/* Resolve hTunnel entries */
	memset(&hints, 0, sizeof(struct addrinfo));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_DGRAM;

	if (getaddrinfo(hTunnel->sIPv4_POP, AYIYA_PORT, &hints, &res) != 0)
	{
		dolog(LOG_ERR, "Couldn't resolve POP IPv4 %s\n", hTunnel->sIPv4_POP);
		return false;
	}
	ressave = res;
        while (res)
        {
		if (res->ai_family != AF_INET)
		{
			res = res->ai_next;
			continue;
		}
		memcpy(&ayiya_ipv4_pop, &((struct sockaddr_in *)res->ai_addr)->sin_addr, 4);
		break;
        }
        freeaddrinfo(ressave);
        if (res == NULL)
        {
		dolog(LOG_ERR, "No valid IPv4 address for POP address %s could be found\n", hTunnel->sIPv4_POP);
		return false;
        }

	memset(&hints, 0, sizeof(struct addrinfo));
	hints.ai_family = AF_INET6;
	if (getaddrinfo(hTunnel->sIPv6_Local, NULL, &hints, &res) != 0)
	{
		dolog(LOG_ERR, "Couldn't resolve Local IPv6 %s\n", hTunnel->sIPv6_Local);
		return false;
	}
	ressave = res;
        while (res)
        {
		if (res->ai_family != AF_INET6)
		{
			res = res->ai_next;
			continue;
		}
		memcpy(&ayiya_ipv6_local, &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr, 16);
		break;
        }
        freeaddrinfo(ressave);
        if (res == NULL)
        {
		dolog(LOG_ERR, "No valid IPv6 address for Local IPv6 address %s could be found\n", hTunnel->sIPv6_Local);
		return false;
        }

	memset(&hints, 0, sizeof(struct addrinfo));
	hints.ai_family = AF_INET6;
	if (getaddrinfo(hTunnel->sIPv6_POP, NULL, &hints, &res) != 0)
	{
		dolog(LOG_ERR, "Couldn't resolve Local IPv6 %s\n", hTunnel->sIPv6_POP);
		return false;
	}
	ressave = res;
        while (res)
        {
		if (res->ai_family != AF_INET6)
		{
			res = res->ai_next;
			continue;
		}
		memcpy(&ayiya_ipv6_pop, &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr, 16);
		break;
        }
        freeaddrinfo(ressave);
        if (res == NULL)
        {
		dolog(LOG_ERR, "No valid IPv6 address for POP IPv6 address %s could be found\n", hTunnel->sIPv6_POP);
		return false;
        }

	if (!hTunnel->sPassword)
	{
		dolog(LOG_ERR, "A password is required for using AYIYA tunnels\n");
		return false;
	}

	/* Generate a SHA1 of the shared secret */
	SHA1_Init(&sha1);
	SHA1_Update(&sha1, (const sha1_byte *)hTunnel->sPassword, (unsigned int)strlen(hTunnel->sPassword));
	SHA1_Final(ayiya_hash, &sha1);

	/* Setup listening socket */
	ayiya_socket = connect_client(hTunnel->sIPv4_POP , AYIYA_PORT, AF_INET, SOCK_DGRAM);
	if (ayiya_socket == -1)
	{
		ayiya_log(LOG_ERR, NULL, 0, "Connection error:: could not create connection to AYIYA server\n");
		return false;
	}

	/* Let AICCU configure the thing */
	aiccu_setup(hTunnel, false);

	/* Show that we have started */
	ayiya_log(LOG_INFO, NULL, 0, "Anything in Anything (%s)\n", AYIYA_VERSION);

	/* Launch a thread for reader */
#ifndef _WIN32
	pthread_create(&thread, NULL, ayiya_reader, NULL);
#else
	h = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ayiya_reader, NULL, 0, &pID);
#endif
	/* The writer takes over this thread and thus keeps running */
	ayiya_writer(NULL);

	return false;
}
