/**********************************************************
 SixXS - Automatic IPv6 Connectivity Configuration Utility
***********************************************************
 Copyright 2003-2005 SixXS - http://www.sixxs.net
***********************************************************
 common/common.h - Common Definitions
***********************************************************
 $Author: jeroen $
 $Id: common.h,v 1.6 2005/01/31 20:32:50 jeroen Exp $
 $Date: 2005/01/31 20:32:50 $
**********************************************************/

#ifndef AICCU_COMMON_H
#define AICCU_COMMON_H "H5K7:W3NDY5UU5N1K1N1C0l3"

#ifdef _DEBUG
#define D(x) x
#else
#define D(x) {}
#endif

#ifndef __OpenBSD__
#define _XOPEN_SOURCE
#endif
#define __STRICT_ANSI__

#include <stdio.h>
#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

/* For MD5 routines */
#define __USE_BSD 1
#include <sys/types.h>

#ifndef _WIN32
	/* Unix Specifics */

#ifndef linux
	#include <netinet/in_systm.h>
#endif

	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <netdb.h>
	#include <unistd.h>
	#include <strings.h>
	#include <syslog.h>
	#include <arpa/inet.h>
	#include <sys/ioctl.h>
	#include <sys/select.h>

	#include <net/if.h>
	#include <netinet/if_ether.h>
#ifdef linux
	#include <netpacket/packet.h>
	#include <linux/if_tun.h>
#endif
	#include <netinet/ip.h>
	#include <netinet/ip6.h>
	#include <netinet/icmp6.h>
	#include <netinet/tcp.h>
	#include <netinet/udp.h>
	#include <netinet/ip_icmp.h>
	#include <sys/ioctl.h>

	#include <pthread.h>

	/*
	 * Windows abstracts sockets to a different
	 * type, as this is actually pretty nice
	 * we'll do it too
	 */
	#ifndef SOCKET
		typedef int SOCKET;
	#endif

	/* closesocket() -> close() on unices */
	#define closesocket close

	/*
	 * Expect a BSD style in6_addr who puts
	 * this between a #ifdef _KERNEL...
	 * Probably against people doing stuff in userspace?
	 */
	#ifndef s6_addr16
	#define s6_addr16 __u6_addr.__u6_addr16
	#endif
#else
	/* Windows Specifics */
	#include <io.h>

	/* Winsock */
	#include <winsock2.h>
	#include <ws2tcpip.h>

	/*
	 * Some weird M$ person thought it was
	 * funny to underscore common functions !?
	 */
	#define snprintf	_snprintf
	#define vsnprintf	_vsnprintf
	#define strcasecmp	_stricmp
	#define strncasecmp	_strnicmp

	/* 
	 * Capitalize this one
	 * Sleep() is in milliseconds
	 */
	#define sleep(x) Sleep(x*1000)

	/* No syslog on Windows */
	#define LOG_DEBUG 1
	#define LOG_ERR 2
	#define LOG_WARNING 3
	#define LOG_INFO 4

	#ifdef __alpha
	typedef unsigned int u_int32_t;
	#else
	typedef unsigned long u_int32_t;
	#endif

	typedef unsigned char u_int8_t;
	typedef unsigned __int16 u_int16_t;

	/* Not available in the Winsock2 includes */
	#define IPPROTO_NONE 59 /* IPv6 no next header */

	#define BIG_ENDIAN 4321
	#define LITTLE_ENDIAN 1234

	#define __BIG_ENDIAN BIG_ENDIAN
	#define __LITTLE_ENDIAN LITTLE_ENDIAN

	/* Fix byte order */
	#ifdef __alpha
	#define __BYTE_ORDER __BIG_ENDIAN
	#define BYTE_ORDER BIG_ENDIAN
	#else
	#define __BYTE_ORDER __LITTLE_ENDIAN
	#define BYTE_ORDER LITTLE_ENDIAN
	#endif

	const char *inet_ntop(int af, const void *src, char *dst, socklen_t cnt);
	int inet_pton(int af, const char *src, void *dst);
#endif /* WIN32 */

#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN  __LITTLE_ENDIAN
#endif
#ifndef BIG_ENDIAN
#define BIG_ENDIAN     __BIG_ENDIAN
#endif
#ifndef PDP_ENDIAN
#define PDP_ENDIAN     __PDP_ENDIAN
#endif
#ifndef BYTE_ORDER
#define BYTE_ORDER     __BYTE_ORDER
#endif

/* Boolean support */
#ifndef bool
#define bool int
#endif
#ifndef false
#define false 0
#endif
#ifndef true
#define true (!false)
#endif

#ifndef offsetof
#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
#endif

/* Include MD5 and SHA1 support */
#include "hash_md5.h"
#include "hash_sha1.h"

/* parseline() rules */
enum pl_ruletype
{
	PLRT_STRING,		/* Offset points to a String (strdup()) */
	PLRT_INTEGER,		/* Offset points to a Integer (unsigned int) */
	PLRT_BOOL,		/* Offset points to a Boolean. */
	PLRT_IPV4,		/* Offset points to a IPv4 address (inet_pton(..., AF_INET)) */
	PLRT_IPV6,		/* Offset points to a IPv6 address (inet_pton(..., AF_INET6)) */
	PLRT_END		/* End of rules */
};

struct pl_rule
{
	const char		*title;
	enum pl_ruletype	type;
	unsigned int		offset;
};

/* Common Functions */
void dologA(int level, const char *fmt, va_list ap);
void dolog(int level, const char *fmt, ...);

#ifndef _WIN32
int sigrunning(int sig);
void savepid(void);
void cleanpid(int i);
#endif

/* Networking functions */
void sock_printf(SOCKET sock, const char *fmt, ...);
int sock_getline(SOCKET sock, char *rbuf, unsigned int rbuflen, unsigned int *filled, char *ubuf, unsigned int ubuflen);
SOCKET connect_client(const char *hostname, const char *service, int family, int socktype);
SOCKET listen_server(const char *description, const char *hostname, const char *service, int family, int socktype);

/* Parsing functions */
unsigned int countfields(char *s);
bool copyfield(char *s, unsigned int n, char *buf, unsigned int buflen);
bool parseline(char *line, const char *split, struct pl_rule *rules, void *data);

/* Convienience */
void MD5String(char *sString, char *sSignature, unsigned int siglen);
bool is_rfc1918(char *ipv4);

#endif /* AICCU_COMMON_H */
